require 'deploy_dply/helper'
require 'fileutils'

module DeployDply
  class Archive

    attr_reader :name, :branch
    attr_writer :skip_depcheck

    def initialize(name, branch: nil, revision: nil)
      @name = name
      @branch = (branch || load_branch).tr("/", "_")
      @revision = revision
      validate
    end

    def created(git: true, gnu_tar: false, &block)
      create_artifacts_dir
      create_tar git, gnu_tar
      instance_eval &block
      add_git_commit_id
      add_revision
      add_archive_name
      depcheck
      compress
    end

    private 

    def helper
      @helper ||= Helper.new
    end

    def create_artifacts_dir
      artifacts_dir = "tmp/build_artifacts"
      FileUtils.mkdir artifacts_dir if not File.directory? artifacts_dir
    end

    def load_branch
      ENV["BUILD_BRANCH"] || ENV["BRANCH_NAME"] || `git rev-parse --abbrev-ref HEAD`.chomp!
    end

    def revision
      @revision ||= ENV["BUILD_NUMBER"]
    end

    def add_git_commit_id
      commit_id = `git rev-parse HEAD`.chomp!
      file = "GIT_COMMIT_ID"
      File.write(file, commit_id)
      add file
    end

    def add_revision
      file = "REVISION"
      File.write(file, revision)
      add file
    end

    def add_archive_name
      file = "ARCHIVE_NAME"
      File.write(file, @name)
      add file
    end

    def add_bundle
      add ".bundle"
      add "vendor/bundle"
    end

    def create_tar(git, gnu_tar)
      tmp_dir = "tmp/app_code"
      if git
        if gnu_tar
          FileUtils.rm_rf tmp_dir if File.exists? tmp_dir
          FileUtils.mkdir tmp_dir
          helper.sh "git archive HEAD | tar xf - -C #{tmp_dir}"
          helper.sh "tar cf #{tar} -C #{tmp_dir} ."
        else
          helper.sh "git archive -o #{tar} HEAD"
        end
      else
        helper.sh "tar -cf #{tar} -T /dev/null"
      end
    ensure
      FileUtils.rm_rf tmp_dir if File.exists? tmp_dir
    end

    def tar
      @tar ||= "tmp/build_artifacts/#{@name}-#{@revision}-#{@branch}.tar"
    end

    def add(file, chdir: nil)
      if chdir
        helper.sh %(tar --hard-dereference -h -C #{chdir} --append -f "#{tar}" "#{file}")
      else
        helper.sh %(tar --hard-dereference -h --append -f "#{tar}" "#{file}")
      end
    end

    def remove(file)
      helper.sh %(tar --delete -f "#{tar}" "#{file}")
    end

    def depcheck
      return if @skip_depcheck
      helper.sh %(drake depcheck #{tar})
    end

    def compress
      helper.sh %(#{gzip} -f -9 "#{tar}")
    end

    def gzip
      File.exists?("/usr/bin/pigz") ? "pigz" : "gzip"
    end

    def validate
      raise "BUILD_NUMBER not specified" if ( !revision || revision.strip.empty?)
      arr = [:name, :branch]
      arr.each do |k|
        v = send k
        v ||= ""
        raise "archive param :#{k} is empty" if v.strip.empty?
      end
    end

  end
end
